︠c9d5d946-6b60-41e0-86d3-f26c1af57e12i︠
%hide
#auto

# this code should be run when the worksheet is loaded!

def mkSymMat(el_name,m,n):
    A = []
    for i in range(0,m):
        A.append([])
        for j in range(0,n):
            A[i].append(var(el_name+str(i+1)+str(j+1)))
    return matrix(A)

def mkSymVec(el_name,m):
    V = []
    for i in range(0,m):
        V.append(var(el_name+str(i+1)))
    return(vector(V))
    
def rfold(op,ls):
    l = len(ls)
    if l>1:
        return op( rfold(op,ls[:(l-1)]) , ls[l-1] )
    else:
        return ls[0]
        
def lfold(op,ls):
    l = len(ls)
    if l>1:
        return op( ls[0], lfold(op,ls[1:l]) )
    else:
        return ls[l-1]
︡062d3ac9-d95a-4fda-93d0-b432be2b8538︡︡
︠08ea2dbf-13d1-4fde-81c6-bbd49892a9e2i︠
%html
<h1>Embedding Flows in a Competition on a Manifold</h1>
<h2>Introduction</h2>
<p>Different dynamics are produced, seemingly by the same underlying system, in different conditions (Huys, others), motivating the question, here in neural systems, of how a homogeneous substrate, such as a sheet of firing rate neurons, can be switched to different dynamics.</p>

︡ece6ddd9-4244-4482-9138-38a58c7d5615︡{"html": "<h1>Embedding Flows in a Competition on a Manifold</h1>\r\n<h2>Introduction</h2>\r\n<p>Different dynamics are produced, seemingly by the same underlying system, in different conditions (Huys, others), motivating the question, here in neural systems, of how a homogeneous substrate, such as a sheet of firing rate neurons, can be switched to different dynamics.</p>"}︡
︠11ef72df-317c-4a19-aeab-468f295f7e32i︠
%hide
︡ee861dcf-ed73-4fa6-ab24-f6293917d017︡{"stdout": "Syntax Error:\n    %hide"}︡
︠3c68b8fc-dfa5-4d89-a232-55bf68202ce8i︠
%html
<h2>Example of Phase Flow Competition<br /></h2>
<p>For a first concrete example, we will take the first four equations 3.5 (and the competition component from 3.4) from Pillai 2008 corresponding to a double Fitzhugh-Nagumo excitator system in which the phase flow composed of $(\xi_1,\xi_2)$ competes with that of $(\xi_3,\xi_4)$:</p>
<p>$ \dot{ \xi_1 } = ( 1-\sum_{i=1}^4 \xi_i^2) \xi_1 + \mu (\xi_2+\xi_1-\xi_1^3/3) \tau - c_1 \xi_1 \sum_i^4 \xi_i^2 $</p>
<p>$ \dot{ \xi_2 } = ( 1-\sum_{i=1}^4 \xi_i^2) \xi_2 -&nbsp; \mu (\xi_1 - a_1 - I_1 - b_1 \xi_2 ) / \tau - c_1&nbsp; \xi_2 \sum_i^4 \xi_i^2 $</p>
<p>$ \dot{ \xi_3 } = ( 1-\sum_{i=1}^4 \xi_i^2) \xi_3 + \mu (\xi_4+\xi_3-\xi_3^3/3) \tau - c_2 \xi_3 \sum_i^4 \xi_i^2 $</p>
<p>$ \dot{ \xi_4 } = ( 1-\sum_{i=1}^4 \xi_i^2) \xi_4 -&nbsp; \mu (\xi_3 - a_2 - I_2 - b_2 \xi_4 ) / \tau - c_2 \xi_4 \sum_i^4 \xi_i^2 $</p>
<p>in which $a_i$, $b_i$, and $I_i$ produce different phase flows, i.e. a combination of monostable, bistable or limit cycle.</p>

︡52c0d641-2b4c-4c90-a05f-f4494b34031d︡{"html": "<h2>Example of Phase Flow Competition<br /></h2>\n<p>For a first concrete example, we will take the first four equations 3.5 (and the competition component from 3.4) from Pillai 2008 corresponding to a double Fitzhugh-Nagumo excitator system in which the phase flow composed of $(\\xi_1,\\xi_2)$ competes with that of $(\\xi_3,\\xi_4)$:</p>\n<p>$ \\dot{ \\xi_1 } = ( 1-\\sum_{i=1}^4 \\xi_i^2) \\xi_1 + \\mu (\\xi_2+\\xi_1-\\xi_1^3/3) \\tau - c_1 \\xi_1 \\sum_i^4 \\xi_i^2 $</p>\n<p>$ \\dot{ \\xi_2 } = ( 1-\\sum_{i=1}^4 \\xi_i^2) \\xi_2 -&nbsp; \\mu (\\xi_1 - a_1 - I_1 - b_1 \\xi_2 ) / \\tau - c_1&nbsp; \\xi_2 \\sum_i^4 \\xi_i^2 $</p>\n<p>$ \\dot{ \\xi_3 } = ( 1-\\sum_{i=1}^4 \\xi_i^2) \\xi_3 + \\mu (\\xi_4+\\xi_3-\\xi_3^3/3) \\tau - c_2 \\xi_3 \\sum_i^4 \\xi_i^2 $</p>\n<p>$ \\dot{ \\xi_4 } = ( 1-\\sum_{i=1}^4 \\xi_i^2) \\xi_4 -&nbsp; \\mu (\\xi_3 - a_2 - I_2 - b_2 \\xi_4 ) / \\tau - c_2 \\xi_4 \\sum_i^4 \\xi_i^2 $</p>\n<p>in which $a_i$, $b_i$, and $I_i$ produce different phase flows, i.e. a combination of monostable, bistable or limit cycle.</p>"}︡
︠7d617687-87de-48d8-9c62-1e250644c38ci︠
%hide
a1, b1, I1 = 0.2, 2, 0
a2, b2, I2 = 1,1,0
c1, c2 = 0, 1
tau, mu = 3, .2

def fsys(t,y):

    # sum of squares of state variables
    sumsq = 0.0
    for i in range(0,4):
        sumsq += y[i]**2
    
    # vector field components, 1st excitator
    dx1 = (1-sumsq)*y[0] + mu*( y[1]+y[0]-(y[0]**3)/3 )*tau - (t/1000)*y[0]*sumsq
    dx2 = (1-sumsq)*y[1] - mu*( y[0] -a1 -I1 -b1*y[1] )/tau - (t/1000)*y[1]*sumsq
    
    # 2nd excitator
    dx3 = (1-sumsq)*y[2] + mu*( y[3]+y[2]-(y[2]**3)/3 )*tau - (1-t/1000)*y[2]*sumsq
    dx4 = (1-sumsq)*y[3] - mu*( y[2] -a2 -I2 -b2*y[3] )/tau - (1-t/1000)*y[3]*sumsq
    
    return [dx1,dx2,dx3,dx4]
    
S = ode_solver( function=fsys, y_0=[1,1,1,1] )
S.ode_solve( t_span=(0,1000), num_points=1000 )

sol = []
t,y = S.solution[0]
sol.append([])
for i in y:
    sol.append([])
for t,y in S.solution:
    sol[0].append(t)
    for j in range(0,len(y)):
        sol[j+1].append(y[j])

t, c1, c2 = [], [],[]
for i in range(1,1001):
    t.append(i*1.0)
    c1.append( i/1000.0 )
    c2.append(1-i/1000.0)
︡b4ca3b23-ac46-47a5-8cca-7267a94d7181︡︡
︠5fe2f47f-a7f1-48b9-87da-dc582efc0177i︠
%hide
import pylab as p
p.close()

p.subplot(211)
p.plot(t,c1,label='$c_1$')
p.plot(t,c2,label='$c_2$')
p.ylabel("$c_i$")
p.legend(loc=7)

p.subplot(212)
p.plot(sol[0],sol[1],label=r"$\xi_1$")
p.plot(sol[0],sol[3],label=r"$\xi_3$")
p.ylabel("activity")
p.xlabel("time")
p.legend(loc=8)
p.savefig("fig1.png")
︡e5115f30-b9ae-4c46-99e2-aeae8067be43︡︡
︠b6eb11d2-7877-49b6-a155-67861d778d18i︠
%html
<h3>Fig 1: Controlling Flows</h3>
<p><span style="font-size: medium;"><span style="font-family: arial,helvetica,sans-serif;">The parameters $c_1$ and $c_2$ control the competition between the $(\xi_1,\xi_2)$ and $(\xi_3,\xi_4)$ dynamics. As $c_i$ is raised (lowered) the $i^{th}$ functional subnetwork's activity is inhibited (enhanced).</span></span></p>
<p>&nbsp;</p>

︡85166a13-3698-4a28-955e-8ffd246d78c9︡{"html": "<h3>Fig 1: Controlling Flows</h3>\r\n<p><span style=\"font-size: medium;\"><span style=\"font-family: arial,helvetica,sans-serif;\">The parameters $c_1$ and $c_2$ control the competition between the $(\\xi_1,\\xi_2)$ and $(\\xi_3,\\xi_4)$ dynamics. As $c_i$ is raised (lowered) the $i^{th}$ functional subnetwork's activity is inhibited (enhanced).</span></span></p>\r\n<p>&nbsp;</p>"}︡
︠d27b3eef-d92d-4ff8-9c1a-0473cf5bf992i︠
%hide
︡fae76264-2e46-44db-8ee7-7b131a5fe1a9︡{"stdout": "Syntax Error:\n    %hide"}︡
︠5c99b74e-89fc-4052-9204-c299b3875441i︠
%html
<h2>Formalities of Functional and Neural Systems<br /></h2>
<p>In terms of order parameters, $\xi_i$ we have</p>
<p>$$ \xi_i = f(\vec{\xi}) \xi_i + \mu g(\vec{\xi}) $$</p>
<p>and in terms firing rate nodes we have</p>
<p>$$ q_i = - q_i + \sum_j [ z_{ij} S(q_j) ] q_i $$</p>
<p>or</p>
<p>$$ \vec{ q } = - \vec{ q } + [\textbf{ Z } \vec{ S } ( \vec{ q } ) &nbsp;] \vec{ q } $$</p>
<p>where $ \textbf{Z} $ is a connectivity matrix. Based on the coordinate transform $\textbf{w}$, from which we have $ \vec{ q } = \textbf{ w } \vec{ \xi } $, $ \vec{ \xi } = \textbf{ w }^\dagger \vec{ q } $ and&nbsp;an adjoint system $ \textbf{w}^\dagger \textbf{w} = \textbf{I} $, we write</p>
<p>$$ \textbf{ w } \vec{ \xi }&nbsp;= [ -1 + \textbf{ Z } \vec{ S } (&nbsp;\textbf{ w } \vec{ \xi }&nbsp;) &nbsp;] \textbf{ w } \vec{ \xi }&nbsp;$$</p>
<p>$$&nbsp;\textbf{w}^\dagger&nbsp;\textbf{ w } \vec{ \xi }&nbsp;= [ -&nbsp;\textbf{w}^\dagger &nbsp;+&nbsp;\textbf{w}^\dagger \textbf{ Z } \vec{ S } (&nbsp;\textbf{ w } \vec{ \xi }&nbsp;) &nbsp;]&nbsp;\textbf{ w } \vec{ \xi }&nbsp;$$</p>
<p>and expand $ \textbf{ Z } $ into $ z_0 + \mu \textbf{ Z } $:&nbsp;</p>
<p>$$ \vec{ \xi }&nbsp;= [ -&nbsp;\textbf{w}^\dagger &nbsp;+&nbsp;\textbf{w}^\dagger (&nbsp;z_0 + \mu \textbf{ Z } ) \vec{ S } (&nbsp;\textbf{ w } \vec{ \xi } ) ]&nbsp;\textbf{ w } \vec{ \xi }&nbsp;$$</p>
<p>$$ \vec{ \xi }&nbsp;= [ -&nbsp;\textbf{w}^\dagger &nbsp;+ z_0 \textbf{w}^\dagger \vec{ S } (&nbsp;\textbf{ w } \vec{ \xi } )&nbsp;+&nbsp;\mu&nbsp;\textbf{w}^\dagger \textbf{ Z }&nbsp;\vec{ S } (&nbsp;\textbf{ w } \vec{ \xi } ) ]&nbsp;\textbf{ w } \vec{ \xi }&nbsp;$$.</p>
<p>At this point, let the neural interaction function (taken to be a sigmoid) be approximated as $ S(q) = a_1 + a_2 q + a_3 q^2 $&nbsp;(vector multiplication is element-wise unless otherwise denoted) and expand the form:&nbsp;</p>
<div>
<p>$$ \vec{ \xi }&nbsp;= [ -&nbsp;\textbf{w}^\dagger &nbsp;+ z_0 \textbf{w}^\dagger ( \vec{ a_1 }&nbsp;+&nbsp;\vec{&nbsp;a_2 } \textbf{ w } \vec{ \xi } +&nbsp;\vec{&nbsp;a_3 } (&nbsp;\textbf{ w } \vec{ \xi } )^2 ) ]&nbsp;\textbf{ w } \vec{ \xi }&nbsp;+ \mu \textbf{w}^\dagger \textbf{ Z } [ \vec{&nbsp;a_1 } +&nbsp;\vec{&nbsp;a_2 } \textbf{ w } \vec{ \xi } + \vec{&nbsp;a_3 } (&nbsp;\textbf{ w } \vec{ \xi } )^2 ] \textbf{ w } \vec{ \xi }&nbsp;$$</p>
<p>In the formulation of functional system dynamics above, flows are a static property of the vector field and each flow has a corresponding coefficient that determines its role in competition with other flows. In the neural network formulation, asymmetries in the connectivity matrix $\textbf{Z}$ produce flows. In order to recruit multiple flows in a neural network, we first assume that, as in the functional form, the capability to produce a particular flow or set of flows is a static property of the vector field, i.e. the connectivity matrix does not change. The neural network analog of the biased competition present in the equations used next is a linear combination of baseline excitability, $I_{j,i}$ in the $i^{th}$ neuron,&nbsp; corresponding to the $j^{th}$ phase flow:</p>
<p>$$ \dot{ q }_i = - ( c_1 I_{1,i} + c_2 I_{2,i} ) - q_i + q_i \sum_j z_{ij} S( q_j ) $$</p>
<p>which after the derivation above takes the form</p>
<p>$$ \dot{\vec{\xi}} = f(\vec{\xi}) \vec{\xi} + \mu g(\vec{\xi}) - \textbf{w}^\dagger ( c_1 \vec{I_1} + c_2 \vec{I_2} ) $$</p>
<p>where $f(\vec{\xi})=\textbf{w}^\dagger [ - 1&nbsp; + z_0 ( a_1 + a_2 \textbf{ w } \vec{ \xi } + a_3 (&nbsp;\textbf{ w } \vec{ \xi } )^2 ) ]&nbsp;\textbf{ w }$ and $g(\vec{\xi})= \textbf{w}^\dagger \textbf{ Z } [ a_1 +&nbsp; a_2 \textbf{ w } \vec{ \xi } + a_3 (\textbf{ w } \vec{ \xi } )^2 ] \textbf{ w } \vec{ \xi } $. Comparing this form to that of the functional dynamics</p>
<p>$$ \dot{\vec{\xi}} = f(\vec{\xi}) \vec{\xi} + \mu g(\vec{\xi}) - \vec{c} \vec{\xi} \sum_i \xi_i^2 $$</p>
<p>we should be able to fit $f(.)$ from the network formulation to that of the functional form in order to create the desired manifold and similarly $g(.)+ \dots$ to produce the flows and flow competition. In the procedure to fit the network formulation, we want to minimize the angle between the functional vector field, $\vec{V}_f (\vec{\xi},\vec{c})$, and the network vector field, $ \vec{V}_n (\vec{\xi},\vec{c})$,</p>
<p>$$ \theta = \| \arccos \frac{ \langle \vec{V}_f, \vec{V}_n \rangle } {\| \vec{V}_f \| \| \vec{V}_n \| } \| $$</p>
<p>($\arccos$ is evaluated element-wise) evaluated over some region of the space spanned by $ (\vec{\xi}, \vec{c})$ by manipulating parameters of the network vector field $ \textbf{Z}$, $z_0$, $a_i$, and $I_{j,i}$. In particular, $z_0$ and $a_i$ will be set when fitting the manifold, while $\textbf{Z}$ and $I_{j,i}$ will be set in order to produce the flows and their competition.</p>
</div>

︡86c37d45-c3cd-456b-bbee-19631c058819︡{"html": "<h2>Formalities of Functional and Neural Systems<br /></h2>\r\n<p>In terms of order parameters, $\\xi_i$ we have</p>\r\n<p>$$ \\xi_i = f(\\vec{\\xi}) \\xi_i + \\mu g(\\vec{\\xi}) $$</p>\r\n<p>and in terms firing rate nodes we have</p>\r\n<p>$$ q_i = - q_i + \\sum_j [ z_{ij} S(q_j) ] q_i $$</p>\r\n<p>or</p>\r\n<p>$$ \\vec{ q } = - \\vec{ q } + [\\textbf{ Z } \\vec{ S } ( \\vec{ q } ) &nbsp;] \\vec{ q } $$</p>\r\n<p>where $ \\textbf{Z} $ is a connectivity matrix. Based on the coordinate transform $\\textbf{w}$, from which we have $ \\vec{ q } = \\textbf{ w } \\vec{ \\xi } $, $ \\vec{ \\xi } = \\textbf{ w }^\\dagger \\vec{ q } $ and&nbsp;an adjoint system $ \\textbf{w}^\\dagger \\textbf{w} = \\textbf{I} $, we write</p>\r\n<p>$$ \\textbf{ w } \\vec{ \\xi }&nbsp;= [ -1 + \\textbf{ Z } \\vec{ S } (&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;) &nbsp;] \\textbf{ w } \\vec{ \\xi }&nbsp;$$</p>\r\n<p>$$&nbsp;\\textbf{w}^\\dagger&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;= [ -&nbsp;\\textbf{w}^\\dagger &nbsp;+&nbsp;\\textbf{w}^\\dagger \\textbf{ Z } \\vec{ S } (&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;) &nbsp;]&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;$$</p>\r\n<p>and expand $ \\textbf{ Z } $ into $ z_0 + \\mu \\textbf{ Z } $:&nbsp;</p>\r\n<p>$$ \\vec{ \\xi }&nbsp;= [ -&nbsp;\\textbf{w}^\\dagger &nbsp;+&nbsp;\\textbf{w}^\\dagger (&nbsp;z_0 + \\mu \\textbf{ Z } ) \\vec{ S } (&nbsp;\\textbf{ w } \\vec{ \\xi } ) ]&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;$$</p>\r\n<p>$$ \\vec{ \\xi }&nbsp;= [ -&nbsp;\\textbf{w}^\\dagger &nbsp;+ z_0 \\textbf{w}^\\dagger \\vec{ S } (&nbsp;\\textbf{ w } \\vec{ \\xi } )&nbsp;+&nbsp;\\mu&nbsp;\\textbf{w}^\\dagger \\textbf{ Z }&nbsp;\\vec{ S } (&nbsp;\\textbf{ w } \\vec{ \\xi } ) ]&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;$$.</p>\r\n<p>At this point, let the neural interaction function (taken to be a sigmoid) be approximated as $ S(q) = a_1 + a_2 q + a_3 q^2 $&nbsp;(vector multiplication is element-wise unless otherwise denoted) and expand the form:&nbsp;</p>\r\n<div>\r\n<p>$$ \\vec{ \\xi }&nbsp;= [ -&nbsp;\\textbf{w}^\\dagger &nbsp;+ z_0 \\textbf{w}^\\dagger ( \\vec{ a_1 }&nbsp;+&nbsp;\\vec{&nbsp;a_2 } \\textbf{ w } \\vec{ \\xi } +&nbsp;\\vec{&nbsp;a_3 } (&nbsp;\\textbf{ w } \\vec{ \\xi } )^2 ) ]&nbsp;\\textbf{ w } \\vec{ \\xi }&nbsp;+ \\mu \\textbf{w}^\\dagger \\textbf{ Z } [ \\vec{&nbsp;a_1 } +&nbsp;\\vec{&nbsp;a_2 } \\textbf{ w } \\vec{ \\xi } + \\vec{&nbsp;a_3 } (&nbsp;\\textbf{ w } \\vec{ \\xi } )^2 ] \\textbf{ w } \\vec{ \\xi }&nbsp;$$</p>\r\n<p>In the formulation of functional system dynamics above, flows are a static property of the vector field and each flow has a corresponding coefficient that determines its role in competition with other flows. In the neural network formulation, asymmetries in the connectivity matrix $\\textbf{Z}$ produce flows. In order to recruit multiple flows in a neural network, we first assume that, as in the functional form, the capability to produce a particular flow or set of flows is a static property of the vector field, i.e. the connectivity matrix does not change. The neural network analog of the biased competition present in the equations used next is a linear combination of baseline excitability, $I_{j,i}$ in the $i^{th}$ neuron,&nbsp; corresponding to the $j^{th}$ phase flow:</p>\r\n<p>$$ \\dot{ q }_i = - ( c_1 I_{1,i} + c_2 I_{2,i} ) - q_i + q_i \\sum_j z_{ij} S( q_j ) $$</p>\r\n<p>which after the derivation above takes the form</p>\r\n<p>$$ \\dot{\\vec{\\xi}} = f(\\vec{\\xi}) \\vec{\\xi} + \\mu g(\\vec{\\xi}) - \\textbf{w}^\\dagger ( c_1 \\vec{I_1} + c_2 \\vec{I_2} ) $$</p>\r\n<p>where $f(\\vec{\\xi})=\\textbf{w}^\\dagger [ - 1&nbsp; + z_0 ( a_1 + a_2 \\textbf{ w } \\vec{ \\xi } + a_3 (&nbsp;\\textbf{ w } \\vec{ \\xi } )^2 ) ]&nbsp;\\textbf{ w }$ and $g(\\vec{\\xi})= \\textbf{w}^\\dagger \\textbf{ Z } [ a_1 +&nbsp; a_2 \\textbf{ w } \\vec{ \\xi } + a_3 (\\textbf{ w } \\vec{ \\xi } )^2 ] \\textbf{ w } \\vec{ \\xi } $. Comparing this form to that of the functional dynamics</p>\r\n<p>$$ \\dot{\\vec{\\xi}} = f(\\vec{\\xi}) \\vec{\\xi} + \\mu g(\\vec{\\xi}) - \\vec{c} \\vec{\\xi} \\sum_i \\xi_i^2 $$</p>\r\n<p>we should be able to fit $f(.)$ from the network formulation to that of the functional form in order to create the desired manifold and similarly $g(.)+ \\dots$ to produce the flows and flow competition. In the procedure to fit the network formulation, we want to minimize the angle between the functional vector field, $\\vec{V}_f (\\vec{\\xi},\\vec{c})$, and the network vector field, $ \\vec{V}_n (\\vec{\\xi},\\vec{c})$,</p>\r\n<p>$$ \\theta = \\| \\arccos \\frac{ \\langle \\vec{V}_f, \\vec{V}_n \\rangle } {\\| \\vec{V}_f \\| \\| \\vec{V}_n \\| } \\| $$</p>\r\n<p>($\\arccos$ is evaluated element-wise) evaluated over some region of the space spanned by $ (\\vec{\\xi}, \\vec{c})$ by manipulating parameters of the network vector field $ \\textbf{Z}$, $z_0$, $a_i$, and $I_{j,i}$. In particular, $z_0$ and $a_i$ will be set when fitting the manifold, while $\\textbf{Z}$ and $I_{j,i}$ will be set in order to produce the flows and their competition.</p>\r\n</div>"}︡
︠07d8a090-88a7-446a-aa4a-86841f1c753di︠
%hide
︡9dcd8b54-0a98-4e34-a865-713622f086b6︡{"stdout": "Syntax Error:\n    %hide"}︡
︠ac0f96df-0d08-4226-b873-2c2a713b8080i︠
%html
<h2>Embedding a Single Flow</h2>
<p>First, we'll attempt to embed a simpler excitator limit-cycle flow into a network of five firing rate neurons. From the above, we have</p>
<p>$$ \dot{ \xi_1 } = ( 1- \xi_1^2 - \xi_2^2) \xi_1 + \mu (\xi_2+\xi_1-\xi_1^3/3) \tau $$</p>
<p>$$ \dot{ \xi_2 } = ( 1-&nbsp; \xi_1^2 - \xi_2^2 ) \xi_2 -&nbsp; \mu (\xi_1 - a_1 - I_1 - b_1 \xi_2 ) / \tau $$</p>
<p>whose behavior we want to map into</p>
<p>$$ \dot{ q_i } = I_i - q_i + q_i \sum_{j=1}^{5} z_{ij} S( q_j ) $$</p>
<p>by fitting the form $ \dot{\vec{\xi}} = f(\vec{\xi}) \vec{\xi} + \mu g(\vec{\xi}) $ (where $f(\vec{\xi})=\textbf{w}^\dagger [ - 1&nbsp; + z_0 ( a_1 + a_2 \textbf{ w } \vec{ \xi } + a_3 (&nbsp;\textbf{ w } \vec{ \xi } )^2 ) ]&nbsp;\textbf{ w }$ and $g(\vec{\xi})= \textbf{w}^\dagger \textbf{ Z } [ a_1 +&nbsp; a_2 \textbf{ w } \vec{ \xi } + a_3 (\textbf{ w } \vec{ \xi } )^2 ] \textbf{ w } \vec{ \xi } $) as shown above. In the following we put subscripts $\xi$ or $q$ to denote the form from which the expression is taken, i.e. $f_\xi (.)$ vs $f_q(.)$.</p>
<p>First, we select a coordinate transform $\textbf{w}$ and one of its possible adjoints:</p>

︡f751d3a7-4be4-47cf-b322-79f3808a3acf︡{"html": "<h2>Embedding a Single Flow</h2>\r\n<p>First, we'll attempt to embed a simpler excitator limit-cycle flow into a network of five firing rate neurons. From the above, we have</p>\r\n<p>$$ \\dot{ \\xi_1 } = ( 1- \\xi_1^2 - \\xi_2^2) \\xi_1 + \\mu (\\xi_2+\\xi_1-\\xi_1^3/3) \\tau $$</p>\r\n<p>$$ \\dot{ \\xi_2 } = ( 1-&nbsp; \\xi_1^2 - \\xi_2^2 ) \\xi_2 -&nbsp; \\mu (\\xi_1 - a_1 - I_1 - b_1 \\xi_2 ) / \\tau $$</p>\r\n<p>whose behavior we want to map into</p>\r\n<p>$$ \\dot{ q_i } = I_i - q_i + q_i \\sum_{j=1}^{5} z_{ij} S( q_j ) $$</p>\r\n<p>by fitting the form $ \\dot{\\vec{\\xi}} = f(\\vec{\\xi}) \\vec{\\xi} + \\mu g(\\vec{\\xi}) $ (where $f(\\vec{\\xi})=\\textbf{w}^\\dagger [ - 1&nbsp; + z_0 ( a_1 + a_2 \\textbf{ w } \\vec{ \\xi } + a_3 (&nbsp;\\textbf{ w } \\vec{ \\xi } )^2 ) ]&nbsp;\\textbf{ w }$ and $g(\\vec{\\xi})= \\textbf{w}^\\dagger \\textbf{ Z } [ a_1 +&nbsp; a_2 \\textbf{ w } \\vec{ \\xi } + a_3 (\\textbf{ w } \\vec{ \\xi } )^2 ] \\textbf{ w } \\vec{ \\xi } $) as shown above. In the following we put subscripts $\\xi$ or $q$ to denote the form from which the expression is taken, i.e. $f_\\xi (.)$ vs $f_q(.)$.</p>\r\n<p>First, we select a coordinate transform $\\textbf{w}$ and one of its possible adjoints:</p>"}︡
︠de6bcaa0-a9ab-45fe-8d8f-487cae0b3baa︠
# matrix with dimensions 5x2
w = matrix( [[1,0], [0,1], [0,0], [0,0], [0,0]] )

# and an adjoint
w_adj = matrix( [[1,0,1,1,0], [0,1,1,1,1]] )
w_adj * w
︡2b45b548-606f-4a77-84e3-20d768f90ea9︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{rr}\n1 & 0 \\\\\n0 & 1\n\\end{array}\\right)</span>"}︡
︠07922636-f452-459d-a390-eca3eb6e9660i︠
%html
<h3>Creating the Manifold<br /></h3>
<p>We describe the functional manifold as</p>
<p>$$ f_\xi (\vec{\xi}) = 1-\xi_1^2-\xi_2^2 $$</p>

︡bd185308-b337-404c-9569-a1a72551762f︡{"html": "<h3>Creating the Manifold<br /></h3>\r\n<p>We describe the functional manifold as</p>\r\n<p>$$ f_\\xi (\\vec{\\xi}) = 1-\\xi_1^2-\\xi_2^2 $$</p>"}︡
︠7abb51de-d6a4-4a8e-adf4-dd9241894214︠
# create symbolic vectors for q and xi space
xi = mkSymVec('xi',2)
q = mkSymVec('q',5)

# parameters
var('z0 a1')
#a1 = mkSymVec('a1',5)
a2 = mkSymVec('a2',5)
a3 = mkSymVec('a3',5)

# xi manifold
f_xi = vector([(1-xi[0]**2-xi[1]**2)*xi[0],xi[1]*(1-xi[0]**2-xi[1]**2)])
f_xi
︡307d59b8-5113-4c0b-ab9b-5760e5aabeba︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(-{(\\xi_{1}^{2} + \\xi_{2}^{2} - 1)} \\xi_{1},-{(\\xi_{1}^{2} + \\xi_{2}^{2} - 1)} \\xi_{2}\\right)</span>"}︡
︠e6d82f97-ae60-4aac-b37d-18538d2df170i︠
%html
<p>&nbsp;</p>
<p>and the neural network manifold</p>
<p>$$ f_q (\vec{\xi}) = \textbf{w}^\dagger [ - 1&nbsp; + z_0 ( a_1 + \vec{a}_2 \textbf{ w } \vec{ \xi } + \vec{a}_3 (&nbsp;\textbf{ w } \vec{ \xi } )^2 ) ] $$</p>

︡f94cb227-dd6c-4b87-887c-20ef5da06ebf︡{"html": "<p>&nbsp;</p>\r\n<p>and the neural network manifold</p>\r\n<p>$$ f_q (\\vec{\\xi}) = \\textbf{w}^\\dagger [ - 1&nbsp; + z_0 ( a_1 + \\vec{a}_2 \\textbf{ w } \\vec{ \\xi } + \\vec{a}_3 (&nbsp;\\textbf{ w } \\vec{ \\xi } )^2 ) ] $$</p>"}︡
︠22228def-a7de-437c-878d-0cc683b15935︠
# q manifold
f_q = []
xi_sq = (w*xi).pairwise_product(w*xi)
for i in range(0,5):
    f_q.append( (-1 + z0*( a1 + a2.dot_product(w*xi) + a3.dot_product(xi_sq)))*((w*xi)[i]))
f_q = w_adj*vector(f_q)
f_q
︡218f2825-2e48-402d-9302-aa32e5b45b7c︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left({({(a_{31} \\xi_{1}^{2} + a_{32} \\xi_{2}^{2} + a_{21} \\xi_{1} + a_{22} \\xi_{2} + a_{1})} z_{0} - 1)} \\xi_{1},{({(a_{31} \\xi_{1}^{2} + a_{32} \\xi_{2}^{2} + a_{21} \\xi_{1} + a_{22} \\xi_{2} + a_{1})} z_{0} - 1)} \\xi_{2}\\right)</span>"}︡
︠b6f99949-876d-4b68-9f1f-4310a070fa4bi︠
%html
<p>Next, we take the angles between the corresponding components of the two vector fields, and sum the square of their errors,</p>
<p>$$ \Theta = \sum_i^2 \theta_i^2 = \sum_i^2 [ \frac{ \int_\xi f_{q,i} f_{\xi,i} d\xi }{ \sqrt{\int_\xi f_{q,i}^2 d\xi} \sqrt{ \int_\xi f_{\xi,i}^2 d\xi} } ]^2$$</p>
<p>where $d\xi$ is a relevant region of the phase space. Elsewhere (Pillai 2008, Huys), fitting has done by numerically evaluating the vector field and finding a numerical angle between high-dimensional vectors. However, because the expressions for the vector fields used here are known, we can integrate to obtain an expression of the sum squared angle in terms of the fitting parameters:&nbsp; (click <span style="color: #999999;"><span style="font-family: courier new,courier;">%hide</span></span> to see code)</p>

︡95656bd2-3bb4-45e1-ab8b-b8b12f3beeba︡{"html": "<p>Next, we take the angles between the corresponding components of the two vector fields, and sum the square of their errors,</p>\r\n<p>$$ \\Theta = \\sum_i^2 \\theta_i^2 = \\sum_i^2 [ \\frac{ \\int_\\xi f_{q,i} f_{\\xi,i} d\\xi }{ \\sqrt{\\int_\\xi f_{q,i}^2 d\\xi} \\sqrt{ \\int_\\xi f_{\\xi,i}^2 d\\xi} } ]^2$$</p>\r\n<p>where $d\\xi$ is a relevant region of the phase space. Elsewhere (Pillai 2008, Huys), fitting has done by numerically evaluating the vector field and finding a numerical angle between high-dimensional vectors. However, because the expressions for the vector fields used here are known, we can integrate to obtain an expression of the sum squared angle in terms of the fitting parameters:&nbsp; (click <span style=\"color: #999999;\"><span style=\"font-family: courier new,courier;\">%hide</span></span> to see code)</p>"}︡
︠db988cf9-7bf8-4d99-b8e7-3fdf025a0c61i︠
%hide
# first define functional dot product wrt vec u, with limits a,b that
# are scalars or vectors w/ length = length(u)
def fdot(f,g,u,a,b):
    prod = f*g
    if isinstance(a,list) and isinstance(b,list):
        for i in range( 0, len(u) ):
            prod = integrate(prod,u[i],a[i],b[i])
    else:
        for i in range( 0, len(u) ):
            prod = integrate(prod,u[i],a,b)
    return(prod)
    
# then a norm
def fnorm(f,u,a,b):
    return sqrt(fdot(f,f,u,a,b))

# now a theta
def ftheta(f,g,u,a,b):
    return acos( fdot(f,g,u,a,b) / (fnorm(f,u,a,b)*fnorm(g,u,a,b)) )

# now, compute Theta over a region of (-10,10)^2
a,b = -2, 2
f_theta = map( lambda f,g: ftheta(f,g,xi,a,b), f_q, f_xi )
f_Theta = sum( map( lambda f: f**2, f_theta ) ) 
f_Theta
︡d6568913-35ce-4a8e-97b0-6dacf84eb88e︡{"stdout": "<html><span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\arccos\\left(-\\frac{2}{3147} \\, \\frac{{({(574 \\, a_{1} - 945 \\, a_{22} + 1064 \\, a_{31} + 1608 \\, a_{32})} z_{0} + {(574 \\, a_{1} + 945 \\, a_{22} + 1064 \\, a_{31} + 1608 \\, a_{32})} z_{0} - 1148)} \\sqrt{\\frac{1049}{35"}︡
︠e13af21c-bac2-435a-8557-88eb23fac6e2i︠
%html
\sqrt{\frac{32}{315} \, {(140 \, {(2 \, a_{1} + 3 \, a_{22})} a_{31} + 168 \, {(3 \, a_{1} + 5 \, a_{22} + 4 \, a_{31})} a_{32} + 105 \, a_{1}^{2} + 315 \, a_{1} a_{22} + 140 \, a_{21}^{2} + 252 \, a_{22}^{2} + 336 \, a_{31}^{2} + 720 \, a_{32}^{2})} z_{0}^{2} + \frac{32}{315} \, {(140 \, {(2 \, a_{1} - 3 \, a_{22})} a_{31} + 168 \, {(3 \, a_{1} - 5 \, a_{22} + 4 \, a_{31})} a_{32} + 105 \, a_{1}^{2} - 315 \, a_{1} a_{22} + 140 \, a_{21}^{2} + 252 \, a_{22}^{2} + 336 \, a_{31}^{2} + 720 \, a_{32}^{2})} z_{0}^{2} - \frac{32}{45} \, {(30 \, a_{1} - 45 \, a_{22} + 40 \, a_{31} + 72 \, a_{32})} z_{0} - \frac{32}{45} \, {(30 \, a_{1} + 45 \, a_{22} + 40 \, a_{31} + 72 \, a_{32})} z_{0} + \frac{64}{3}}}\right)^{2} + \arccos\left(-\frac{4}{3147} \, \frac{{({(287 \, a_{1} - 357 \, a_{22} + 804 \, a_{31} + 532 \, a_{32})} z_{0} + {(287 \, a_{1} + 357 \, a_{22} + 804 \, a_{31} + 532 \, a_{32})} z_{0} - 574)} \sqrt{\frac{1049}{35}}}{\sqrt{\frac{32}{315} \, {(504 \, {(a_{1} + a_{22})} a_{31} + 28 \, {(10 \, a_{1} + 15 \, a_{22} + 24 \, a_{31})} a_{32} + 105 \, a_{1}^{2} + 210 \, a_{1} a_{22} + 252 \, a_{21}^{2} + 140 \, a_{22}^{2} + 720 \, a_{31}^{2} + 336 \, a_{32}^{2})} z_{0}^{2} + \frac{32}{315} \, {(504 \, {(a_{1} - a_{22})} a_{31} + 28 \, {(10 \, a_{1} - 15 \, a_{22} + 24 \, a_{31})} a_{32} + 105 \, a_{1}^{2} - 210 \, a_{1} a_{22} + 252 \, a_{21}^{2} + 140 \, a_{22}^{2} + 720 \, a_{31}^{2} + 336 \, a_{32}^{2})} z_{0}^{2} - \frac{64}{45} \, {(15 \, a_{1} - 15 \, a_{22} + 36 \, a_{31} + 20 \, a_{32})} z_{0} - \frac{64}{45} \, {(15 \, a_{1} + 15 \, a_{22} + 36 \, a_{31} + 20 \, a_{32})} z_{0} + \frac{64}{3}}}\right)^{2}</span></html>
}}}

<p>Next, we can minimize the expression for the sum squared angle to a value near zero (using a guess for starting):</p>

︡8d63b9b5-534f-475b-809d-e2132cce142e︡{"html": "\\sqrt{\\frac{32}{315} \\, {(140 \\, {(2 \\, a_{1} + 3 \\, a_{22})} a_{31} + 168 \\, {(3 \\, a_{1} + 5 \\, a_{22} + 4 \\, a_{31})} a_{32} + 105 \\, a_{1}^{2} + 315 \\, a_{1} a_{22} + 140 \\, a_{21}^{2} + 252 \\, a_{22}^{2} + 336 \\, a_{31}^{2} + 720 \\, a_{32}^{2})} z_{0}^{2} + \\frac{32}{315} \\, {(140 \\, {(2 \\, a_{1} - 3 \\, a_{22})} a_{31} + 168 \\, {(3 \\, a_{1} - 5 \\, a_{22} + 4 \\, a_{31})} a_{32} + 105 \\, a_{1}^{2} - 315 \\, a_{1} a_{22} + 140 \\, a_{21}^{2} + 252 \\, a_{22}^{2} + 336 \\, a_{31}^{2} + 720 \\, a_{32}^{2})} z_{0}^{2} - \\frac{32}{45} \\, {(30 \\, a_{1} - 45 \\, a_{22} + 40 \\, a_{31} + 72 \\, a_{32})} z_{0} - \\frac{32}{45} \\, {(30 \\, a_{1} + 45 \\, a_{22} + 40 \\, a_{31} + 72 \\, a_{32})} z_{0} + \\frac{64}{3}}}\\right)^{2} + \\arccos\\left(-\\frac{4}{3147} \\, \\frac{{({(287 \\, a_{1} - 357 \\, a_{22} + 804 \\, a_{31} + 532 \\, a_{32})} z_{0} + {(287 \\, a_{1} + 357 \\, a_{22} + 804 \\, a_{31} + 532 \\, a_{32})} z_{0} - 574)} \\sqrt{\\frac{1049}{35}}}{\\sqrt{\\frac{32}{315} \\, {(504 \\, {(a_{1} + a_{22})} a_{31} + 28 \\, {(10 \\, a_{1} + 15 \\, a_{22} + 24 \\, a_{31})} a_{32} + 105 \\, a_{1}^{2} + 210 \\, a_{1} a_{22} + 252 \\, a_{21}^{2} + 140 \\, a_{22}^{2} + 720 \\, a_{31}^{2} + 336 \\, a_{32}^{2})} z_{0}^{2} + \\frac{32}{315} \\, {(504 \\, {(a_{1} - a_{22})} a_{31} + 28 \\, {(10 \\, a_{1} - 15 \\, a_{22} + 24 \\, a_{31})} a_{32} + 105 \\, a_{1}^{2} - 210 \\, a_{1} a_{22} + 252 \\, a_{21}^{2} + 140 \\, a_{22}^{2} + 720 \\, a_{31}^{2} + 336 \\, a_{32}^{2})} z_{0}^{2} - \\frac{64}{45} \\, {(15 \\, a_{1} - 15 \\, a_{22} + 36 \\, a_{31} + 20 \\, a_{32})} z_{0} - \\frac{64}{45} \\, {(15 \\, a_{1} + 15 \\, a_{22} + 36 \\, a_{31} + 20 \\, a_{32})} z_{0} + \\frac{64}{3}}}\\right)^{2}</span></html>\n}}}\n\n<p>Next, we can minimize the expression for the sum squared angle to a value near zero (using a guess for starting):</p>"}︡
︠932f712f-d59c-4325-add7-f3bb6f71bfd7︠
tmin = list(minimize(f_Theta,[0,0,0,0,1,1]))
︡64ed5a8a-29f4-4f63-951a-c55f7929772f︡{"stdout": "Optimization terminated successfully.\n         Current function value: 0.000000\n         Iterations: 19\n         Function evaluations: 22\n         Gradient evaluations: 22"}︡
︠59807dd5-6830-4d35-9c07-ab46f70d4a2di︠
%html
<p>Just to check, we evalute the expression at the parameters found:</p>

︡7b04ddce-8fe8-4989-b691-fb6076610115︡{"html": "<p>Just to check, we evalute the expression at the parameters found:</p>"}︡
︠0b544cf3-48e3-4794-8320-84fe0aa74d07︠
print 'f_Theta = %g' % f_Theta.subs(a1=tmin[0],a21=tmin[1],a22=tmin[2],a31=tmin[3],a32=tmin[4],z0=tmin[5]).numerical_approx()
︡4f3f47e3-6596-44ec-a12f-0086ff91888f︡{"stdout": "f_Theta = 7.80465e-12"}︡
︠daf9e83c-10d7-43cc-9b6e-a2a17827e2c8i︠
%html
<p>Now, we verify the foregoing method works by sampling the two vector fields over a region of space, and compute the angle of the high-dimensional vector (click <span style="color: #999999;"><span style="font-family: courier new,courier;">%hide</span></span> to see code).</p>

︡16fd4d80-24b7-411b-ad8e-fb6f69a54aae︡{"html": "<p>Now, we verify the foregoing method works by sampling the two vector fields over a region of space, and compute the angle of the high-dimensional vector (click <span style=\"color: #999999;\"><span style=\"font-family: courier new,courier;\">%hide</span></span> to see code).</p>"}︡
︠6450bbc6-127b-46cb-8123-89e48f763252i︠
%hide

# create fast versions of the two vector fields
fast_f_xi = map(lambda f:fast_float(f,'xi1','xi2'),f_xi)
fast_f_q = map(lambda f:fast_float(f,'xi1','xi2'), map( lambda f: f.subs(a1=tmin[0],a21=tmin[1],a22=tmin[2],a31=tmin[3],a32=tmin[4],z0=tmin[5]), f_q ) )

# evaluate vec fields over and npts x npts grid
npts = 40
n_Theta_args1 = [ 0.0, 0.0, 0.0 ]
n_Theta_args2 = [ 0.0, 0.0, 0.0 ]

import numpy as np
n_xi1, n_xi2 = np.linspace(a,b,npts), np.linspace(a,b,npts)
vf_q_u, vf_q_v, vf_xi_u, vf_xi_v = np.zeros([npts,npts]), np.zeros([npts,npts]), np.zeros([npts,npts]), np.zeros([npts,npts])

for i in range(0,npts):
    for j in range(0,npts):
        vf_q_u[i,j] = fast_f_q[0]( n_xi1[i], n_xi2[j] )
        vf_q_v[i,j] = fast_f_q[1]( n_xi1[i], n_xi2[j] )
        vf_xi_u[i,j] = fast_f_xi[0]( n_xi1[i], n_xi2[j] )
        vf_xi_v[i,j] = fast_f_xi[1]( n_xi1[i], n_xi2[j] )
        
for i in range(0,npts):
    for j in range(0,npts): 
        n_Theta_args1[0] += vf_q_u[i,j]*vf_xi_u[i,j] 
        n_Theta_args2[0] += vf_q_v[i,j]*vf_xi_v[i,j]
        n_Theta_args1[1] += vf_q_u[i,j]**2 
        n_Theta_args2[1] += vf_q_v[i,j]**2
        n_Theta_args1[2] += vf_xi_u[i,j]**2 
        n_Theta_args2[2] += vf_xi_v[i,j]**2

n_Theta = [ acos( n_Theta_args1[0] / (sqrt( n_Theta_args1[1] )*sqrt( n_Theta_args1[2] )) ),
            acos( n_Theta_args2[0] / (sqrt( n_Theta_args2[1] )*sqrt( n_Theta_args2[2] )) ) ]
            
print 'Theta = %g' % sqrt( sum( map( lambda x: x**2, n_Theta ) ) )
︡8c494c5d-3a85-42ab-bc57-45126ac8c2b7︡{"stdout": "Theta = 2.5922e-06"}︡
︠5b4da578-abad-481b-94ed-786acec39608i︠
%html
<p>Close enough.</p>
<h3>Creating the Flow</h3>
<p>The next step is to fit</p>
<p>$$ g_q(\vec{\xi}) = \textbf{w}^\dagger \textbf{ Z } [ a_1 +&nbsp; \vec{a}_2 \textbf{ w } \vec{ \xi } + \vec{a}_3 (\textbf{ w } \vec{ \xi } )^2 ] \textbf{ w } \vec{ \xi }$$</p>
<p>to</p>
<p>$$&nbsp; g_\xi(\vec{\xi})=\begin{pmatrix} (\xi_2+\xi_1-\xi_1^3/3) \tau&nbsp; \\ (\xi_1 - \alpha - \beta \xi_2 ) / \tau&nbsp; \end{pmatrix} $$</p>

<p>As before, we create the functional flow</p>

︡500f6323-c8fd-4aaf-a0b6-200548fce3cd︡{"html": "<p>Close enough.</p>\r\n<h3>Creating the Flow</h3>\r\n<p>The next step is to fit</p>\r\n<p>$$ g_q(\\vec{\\xi}) = \\textbf{w}^\\dagger \\textbf{ Z } [ a_1 +&nbsp; \\vec{a}_2 \\textbf{ w } \\vec{ \\xi } + \\vec{a}_3 (\\textbf{ w } \\vec{ \\xi } )^2 ] \\textbf{ w } \\vec{ \\xi }$$</p>\r\n<p>to</p>\r\n<p>$$&nbsp; g_\\xi(\\vec{\\xi})=\\begin{pmatrix} (\\xi_2+\\xi_1-\\xi_1^3/3) \\tau&nbsp; \\\\ (\\xi_1 - \\alpha - \\beta \\xi_2 ) / \\tau&nbsp; \\end{pmatrix} $$</p>\r\n\r\n<p>As before, we create the functional flow</p>"}︡
︠a879b9b8-4932-4373-9969-dba37ea40990︠
var('alpha beta tau')
g_xi = vector( [ (xi2+xi1-(xi1**3)/3)/tau, -(xi1-alpha-beta*xi2)/tau ])
g_xi
︡71df39cd-8ac7-4d13-ac7c-c82bfc8f0a62︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(-\\frac{1}{3} \\, \\frac{{(\\xi_{1}^{3} - 3 \\, \\xi_{1} - 3 \\, \\xi_{2})}}{\\tau},\\frac{{(\\beta \\xi_{2} + \\alpha - \\xi_{1})}}{\\tau}\\right)</span>"}︡
︠c05d3d45-0e60-4ce0-90a4-830c49842d20i︠
%html
<p>and the neural flow</p>

︡19c87038-1aae-4c8b-aded-222c1f681a73︡{"html": "<p>and the neural flow</p>"}︡
︠6248aed1-42bd-4693-915d-4ee95530ec50︠
Z = mkSymMat('z',5,5)
B = mkSymVec('b',5)
g_q = []
for i in range(0,5):
    g_q.append( a1 + a2.dot_product(w*xi) + a3.dot_product( (w*xi).pairwise_product(w*xi) ) )
g_q = w_adj * Z * vector(g_q).pairwise_product(w*xi) + w_adj*B
g_q
︡42b68bcf-98ea-4395-8ca4-bdc3f1a8a125︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left({(z_{12} + z_{32} + z_{42})} {(a_{31} \\xi_{1}^{2} + a_{32} \\xi_{2}^{2} + a_{21} \\xi_{1} + a_{22} \\xi_{2} + a_{1})} \\xi_{2} + {(z_{11} + z_{31} + z_{41})} {(a_{31} \\xi_{1}^{2} + a_{32} \\xi_{2}^{2} + a_{21} \\xi_{1} + a_{22} \\xi_{2} + a_{1})} \\xi_{1} + b_{1} + b_{3} + b_{4},{(z_{22} + z_{32} + z_{42} + z_{52})} {(a_{31} \\xi_{1}^{2} + a_{32} \\xi_{2}^{2} + a_{21} \\xi_{1} + a_{22} \\xi_{2} + a_{1})} \\xi_{2} + {(z_{21} + z_{31} + z_{41} + z_{51})} {(a_{31} \\xi_{1}^{2} + a_{32} \\xi_{2}^{2} + a_{21} \\xi_{1} + a_{22} \\xi_{2} + a_{1})} \\xi_{1} + b_{2} + b_{3} + b_{4} + b_{5}\\right)</span>"}︡
︠e8f128e9-186c-4960-bc6f-f8c338e9c837i︠
%html
<p>substitute our previous values for $a_{ij}$ and set $\alpha$, $\beta$ and $\tau$ in the functional flow,</p>

︡404434e6-3b3e-4b10-8b20-7adac24df641︡{"html": "<p>substitute our previous values for $a_{ij}$ and set $\\alpha$, $\\beta$ and $\\tau$ in the functional flow,</p>"}︡
︠e7c8ad21-77b3-44fd-b76c-f43df6a247b9︠
g_q_ = map( lambda f: f.subs(a1=tmin[0],a21=tmin[1],a22=tmin[2],a31=tmin[3],a32=tmin[4]), g_q )
g_xi_ = map( lambda f: f.subs(alpha=.2,beta=2,tau=3), g_xi)
︡10fbf0cb-2c2d-4489-a356-d4f46f1687c1︡︡
︠45b1c910-e420-434b-b021-ce8258c75901i︠
%html
<p>compute $\Theta$,</p>

︡759e4453-b6dd-4b8e-98c0-fd4abd4d2e5c︡{"html": "<p>compute $\\Theta$,</p>"}︡
︠e6aa849d-939a-4f3c-bb39-3777dcb907e5︠
g_theta = map( lambda f,g: ftheta(f,g,xi,a,b), g_q_, g_xi_ )
g_Theta = sum( map( lambda f: f**2, g_theta ) )
︡0f398de1-860a-4900-9edc-f597bcefe85f︡︡
︠06961351-fbbd-4291-be61-dd55e4f7930bi︠
%html
<p>minimize using free variables</p>

︡bec65558-1e8d-48d1-b67f-86634147a7a3︡{"html": "<p>minimize using free variables</p>"}︡
︠aee87f51-fdd2-416d-a3b6-90a88799e800︠
g_varlist = g_Theta.variables()
g_varlist, len(g_varlist)
︡95476a2b-f0be-498d-b6ea-0c947f2ddb54︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}((b1, b2, b3, b4, b5, z11, z12, z21, z22, z31, z32, z41, z42, z51, z52), 15)</span>"}︡
︠8d8406d7-ad13-4c3a-91e6-1acc937b2c2f︠
g_ic = []
for i in range(0,len(g_varlist)):
    g_ic.append(-.2)
minimize(g_Theta,g_ic)
︡2e7093ab-c23e-4339-9502-e66afec549b9︡{"stdout": "Optimization terminated successfully.\n         Current function value: 0.713578\n         Iterations: 44\n         Function evaluations: 45\n         Gradient evaluations: 45"}︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(-2.58441011722,3.67728797475,1.29287785753,1.29287785753,3.67728797475,-1.66308902101,-3.62578243786,1.93102203328,-0.686072587586,0.467933012272,-4.11185502545,0.467933012272,-4.11185502545,1.93102203328,-0.686072587586\\right)</span>"}︡
︠1a8457dd-abfd-45da-82e2-207579dd60a6︠
var('t1 t2 t3')
eq1 = g_q_[0].expand() == g_xi_[0]
eq1.subs(z12=0,z32=0,z31=0,z41=0)
︡deedd9dd-185d-448d-a782-a38a190eef51︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}-2.88384694956 \\, \\xi_{1}^{3} z_{11} - 2.88384694956 \\, \\xi_{1}^{2} \\xi_{2} z_{42} - 2.883849661 \\, \\xi_{1} \\xi_{2}^{2} z_{11} - 2.883849661 \\, \\xi_{2}^{3} z_{42} + 3.16144543614 \\, \\xi_{1} z_{11} + 3.16144543614 \\, \\xi_{2} z_{42} = -\\frac{1}{9} \\, \\xi_{1}^{3} + \\frac{1}{3} \\, \\xi_{1} + \\frac{1}{3} \\, \\xi_{2}</span>"}︡
︠acff12d7-5aac-4de4-9a75-74bd42e56c4a︠
g_q_[0].variables()
︡77a1f3c1-9820-4934-9d0a-f76748483d4c︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\xi_{1}, \\xi_{2}, z_{11}, z_{12}, z_{31}, z_{32}, z_{41}, z_{42}\\right)</span>"}︡
︠c85843ce-93a7-45fc-80a9-c6fff67813abi︠
%html
<h1>Most of that was wrong, so Now, I'm gonna try again to translate the neural form to the functional form</h1>

︡ef7be52d-fde8-4ff3-a1e1-2936260a3896︡{"html": "<h1>Most of that was wrong, so Now, I'm gonna try again to translate the neural form to the functional form</h1>"}︡
︠ebdfa1c5-4ebd-4771-aa9d-0d91fc0437ea︠
N = 4
M = 2

q = mkSymVec('q',N)
xi = mkSymVec('xi',M)

w = mkSymMat('w',N,M)
wi = mkSymMat('wi',M,N) # inverse
Z = mkSymMat('z',N,N)
a = mkSymVec('a',N)
b = mkSymVec('b',N)
c = mkSymVec('c',N)
I = mkSymVec('I',N)
︡e6a0ba67-7784-46ac-8ddc-162b678dcda6︡︡
︠80f90f5f-09c3-464b-a3ce-c73183cd9d13︠
# ddt q_i = q_i ( -1 + sum_{j=1}^N ( z_{ij} S( q_j ) ) )
input_vec = Z*(a+b.pairwise_product(q)+c.pairwise_product(q.pairwise_product(q))).transpose()
vq = []
for i in range(0,N):
    vq.append( I[i] + q[i]*(-1 + input_vec[i][0]) )
vector(vq).transpose()
︡0b508fbd-0b6d-41fd-a0b1-36709a9309c9︡{"stdout": "[((c4*q4^2 + b4*q4 + a4)*z14 + (c3*q3^2 + b3*q3 + a3)*z13 + (c2*q2^2 + b2*q2 + a2)*z12 + (c1*q1^2 + b1*q1 + a1)*z11 - 1)*q1 + I1]\n[((c4*q4^2 + b4*q4 + a4)*z24 + (c3*q3^2 + b3*q3 + a3)*z23 + (c2*q2^2 + b2*q2 + a2)*z22 + (c1*q1^2 + b1*q1 + a1)*z21 - 1)*q2 + I2]\n[((c4*q4^2 + b4*q4 + a4)*z34 + (c3*q3^2 + b3*q3 + a3)*z33 + (c2*q2^2 + b2*q2 + a2)*z32 + (c1*q1^2 + b1*q1 + a1)*z31 - 1)*q3 + I3]\n[((c4*q4^2 + b4*q4 + a4)*z44 + (c3*q3^2 + b3*q3 + a3)*z43 + (c2*q2^2 + b2*q2 + a2)*z42 + (c1*q1^2 + b1*q1 + a1)*z41 - 1)*q4 + I4]"}︡
︠32bf97dc-7ee7-4663-bc77-31038173a05d︠
qsub = w*xi.transpose(); qsub
︡f56f4076-eeb5-414c-b70b-821b287a3787︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{r}\nw_{11} \\xi_{1} + w_{12} \\xi_{2} \\\\\nw_{21} \\xi_{1} + w_{22} \\xi_{2} \\\\\nw_{31} \\xi_{1} + w_{32} \\xi_{2} \\\\\nw_{41} \\xi_{1} + w_{42} \\xi_{2}\n\\end{array}\\right)</span>"}︡
︠f8ca83ee-5eb6-4685-a5ee-06f2b7c57d26︠
vxiq = []
for i in range(0,N):
    vxiq.append( vq[i].subs( q1=qsub[0][0], q2=qsub[1][0], q3=qsub[2][0], q4=qsub[3][0] ) )
vector(vxiq).transpose()
︡b5ae531c-055e-4ed3-8295-bfdc8539bf2b︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{r}\n{(w_{11} \\xi_{1} + w_{12} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{14} - 1)} + I_{1} \\\\\n{(w_{21} \\xi_{1} + w_{22} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{24} - 1)} + I_{2} \\\\\n{(w_{31} \\xi_{1} + w_{32} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{31} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{32} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{33} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{34} - 1)} + I_{3} \\\\\n{(w_{41} \\xi_{1} + w_{42} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{41} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{42} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{43} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{44} - 1)} + I_{4}\n\\end{array}\\right)</span>"}︡
︠cce351be-bf17-43ce-ab1f-53ecbaddf639︠
vxi = wi*(vector(vxiq)).transpose(); vxi
︡d7f3ef07-75c8-442e-92c9-a60a5ae8dd88︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{r}\n{({(w_{11} \\xi_{1} + w_{12} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{14} - 1)} + I_{1})} \\mbox{wi}_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{24} - 1)} + I_{2})} \\mbox{wi}_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{31} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{32} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{33} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{34} - 1)} + I_{3})} \\mbox{wi}_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{41} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{42} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{43} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{44} - 1)} + I_{4})} \\mbox{wi}_{14} \\\\\n{({(w_{11} \\xi_{1} + w_{12} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{14} - 1)} + I_{1})} \\mbox{wi}_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{24} - 1)} + I_{2})} \\mbox{wi}_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{31} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{32} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{33} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{34} - 1)} + I_{3})} \\mbox{wi}_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + {(w_{11} \\xi_{1} + w_{12} \\xi_{2})} b_{1} + a_{1})} z_{41} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + {(w_{21} \\xi_{1} + w_{22} \\xi_{2})} b_{2} + a_{2})} z_{42} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + {(w_{31} \\xi_{1} + w_{32} \\xi_{2})} b_{3} + a_{3})} z_{43} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + {(w_{41} \\xi_{1} + w_{42} \\xi_{2})} b_{4} + a_{4})} z_{44} - 1)} + I_{4})} \\mbox{wi}_{24}\n\\end{array}\\right)</span>"}︡
︠32a259fa-d71f-4ce0-b5f0-d5f7c09615e3︠
vxis = []
for i in range(0,M):
    vxis.append( vxi[i][0].subs( b1=0, b2=0, b3=0, b4=0) )
vector(vxis).transpose()
︡35441d17-c6e5-420a-9026-6623d5f0e0a7︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{r}\n{({(w_{11} \\xi_{1} + w_{12} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{14} - 1)} + I_{1})} \\mbox{wi}_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{24} - 1)} + I_{2})} \\mbox{wi}_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{31} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{32} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{33} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{34} - 1)} + I_{3})} \\mbox{wi}_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{41} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{42} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{43} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{44} - 1)} + I_{4})} \\mbox{wi}_{14} \\\\\n{({(w_{11} \\xi_{1} + w_{12} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{11} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{12} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{13} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{14} - 1)} + I_{1})} \\mbox{wi}_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{21} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{24} - 1)} + I_{2})} \\mbox{wi}_{22} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{31} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{32} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{33} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{34} - 1)} + I_{3})} \\mbox{wi}_{23} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})} {({({(w_{11} \\xi_{1} + w_{12} \\xi_{2})}^{2} c_{1} + a_{1})} z_{41} + {({(w_{21} \\xi_{1} + w_{22} \\xi_{2})}^{2} c_{2} + a_{2})} z_{42} + {({(w_{31} \\xi_{1} + w_{32} \\xi_{2})}^{2} c_{3} + a_{3})} z_{43} + {({(w_{41} \\xi_{1} + w_{42} \\xi_{2})}^{2} c_{4} + a_{4})} z_{44} - 1)} + I_{4})} \\mbox{wi}_{24}\n\\end{array}\\right)</span>"}︡
︠2e5fe921-2dbf-4eba-a695-d89224942d8f︠
W = matrix( [[1,0],[0,1],[1,0],[0,1]] )/sqrt(N/2)
Wi = matrix( [[1,0,1,0],[0,1,0,1]] )/sqrt(N/2)
Wi*W
︡49be6eb0-2fdd-4949-9dab-e47347c52a33︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{rr}\n1 & 0 \\\\\n0 & 1\n\\end{array}\\right)</span>"}︡
︠2f6601eb-345a-4963-a6c9-46bb4fae8278︠
vxi2 = []
for i in range(0,M):
    temp = vxis[i].collect( [xi1,xi2][i] ).subs( c1=-4/N^2, c2=-4/N^2, c3=-4/N^2, c4=-4/N^2, a1=2/N, a2=2/N, a3=2/N, a4=2/N, z14=0, z32=0, z34=0, z13=0, z31=0, z33=0, z23=0, z41=0, z43=0, z24=0, z42=0, z44=0).expand()
    for j1 in range(0,N):
        for j2 in range(0,M):
            temp = eval('temp.subs( w' + str(j1+1) + str(j2+1) + ' = W[' + str(j1) + ',' + str(j2) + '], wi' + str(j2+1) + str(j1+1) + ' = Wi[' + str(j2) + ','+ str(j1)+'] )')
    vxi2.append(temp)
vector(vxi2).transpose()
︡8fbc27fe-52b1-4930-bec7-f4afcaec49f4︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{r}\n-\\frac{1}{16} \\, \\xi_{1}^{3} z_{11} - \\frac{1}{16} \\, \\xi_{1} \\xi_{2}^{2} z_{12} + \\frac{1}{4} \\, \\xi_{1} z_{11} + \\frac{1}{4} \\, \\xi_{1} z_{12} + \\frac{1}{2} \\, \\sqrt{2} I_{1} + \\frac{1}{2} \\, \\sqrt{2} I_{3} - \\xi_{1} \\\\\n-\\frac{1}{16} \\, \\xi_{1}^{2} \\xi_{2} z_{21} - \\frac{1}{16} \\, \\xi_{2}^{3} z_{22} + \\frac{1}{4} \\, \\xi_{2} z_{21} + \\frac{1}{4} \\, \\xi_{2} z_{22} + \\frac{1}{2} \\, \\sqrt{2} I_{2} + \\frac{1}{2} \\, \\sqrt{2} I_{4} - \\xi_{2}\n\\end{array}\\right)</span>"}︡
︠f1ec9b32-6ae1-4263-b7bf-8f7a54640f39i︠
%html
<p>$ \dot{ \xi_1 } = ( 1-\sum_{i=1}^4 \xi_i^2) \xi_1 + \mu (\xi_2+\xi_1-\xi_1^3/3) \tau - c_1 \xi_1 \sum_i^4 \xi_i^2 $</p>
<p>$ \dot{ \xi_2 } = ( 1-\sum_{i=1}^4 \xi_i^2) \xi_2 + \mu ( - \xi_1 + a_1 + I_1 + b_1 \xi_2 ) / \tau - c_1&nbsp; \xi_2 \sum_i^4 \xi_i^2 $</p>

︡84a62ed1-18cc-4c5f-a4a5-ded3d1d7f979︡{"html": "<p>$ \\dot{ \\xi_1 } = ( 1-\\sum_{i=1}^4 \\xi_i^2) \\xi_1 + \\mu (\\xi_2+\\xi_1-\\xi_1^3/3) \\tau - c_1 \\xi_1 \\sum_i^4 \\xi_i^2 $</p>\n<p>$ \\dot{ \\xi_2 } = ( 1-\\sum_{i=1}^4 \\xi_i^2) \\xi_2 + \\mu ( - \\xi_1 + a_1 + I_1 + b_1 \\xi_2 ) / \\tau - c_1&nbsp; \\xi_2 \\sum_i^4 \\xi_i^2 $</p>"}︡
︠0c6da80e-03da-4b43-9e59-9a67297c0d59︠
var('alpha tau')
vxi3 = vector([vxi2[0].subs(I1=0,I3=0,z12=1,z11=8), vxi2[1].subs(I2=0,I4=2*alpha/sqrt(2), z22=0, z21=4)]).transpose(); vxi3
︡78df3e5b-5fd8-46b7-bc66-0e8f0dfe2c8b︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\begin{array}{r}\n-\\frac{1}{2} \\, \\xi_{1}^{3} - \\frac{1}{16} \\, \\xi_{1} \\xi_{2}^{2} + \\frac{5}{4} \\, \\xi_{1} \\\\\n-\\frac{1}{4} \\, \\xi_{1}^{2} \\xi_{2} + \\alpha\n\\end{array}\\right)</span>"}︡
︠5db10edf-230e-424c-9d20-b8513a19be4b︠
solve( vxi3[0][0]==0, xi2 ), solve( vxi3[1][0]==0, xi2 )
︡e774113e-08ea-4d4a-a470-a5457130033d︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(\\text{[\nxi2 == -2*sqrt(-2*xi1^2 + 5),\nxi2 == 2*sqrt(-2*xi1^2 + 5)\n]}, \\text{[\nxi2 == 4*alpha/xi1^2\n]}\\right)</span>"}︡
︠458b4d5f-c171-4b58-bfc2-b7e4ca656ba1︠
vxi3[0]
︡152cf3fc-c0e7-42bf-97d0-f955ab537c9e︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}\\left(-\\frac{1}{2} \\, \\xi_{1}^{3} - \\frac{1}{16} \\, \\xi_{1} \\xi_{2}^{2} + \\frac{5}{4} \\, \\xi_{1}\\right)</span>"}︡
︠72872d0c-b79f-4861-83cb-a02ee9dabffe︠
import numpy as np

vf1 = ((1-xi1^2-xi2^2)*xi1 + vxi3[0][0])._fast_float_(xi1,xi2)
vf2 = ((1-xi1^2-xi2^2)*xi2 + vxi3[1][0])._fast_float_(xi1,xi2,alpha)

test2 = ode_solver( function= lambda t,y: [vf1(y[0],y[1]),vf2(y[0],y[1],.3)], y_0=[.3,-.3] )
test2.ode_solve( t_span=(0,200), num_points=200 )

sol = []
t,y = test2.solution[0]
sol.append([])
for i in y:
    sol.append([])
for t,y in test2.solution:
    sol[0].append(t)
    for j in range(0,len(y)):
        sol[j+1].append(y[j])

import pylab as p
p.close()
p.plot(sol[0],sol[1],sol[0],sol[2])
p.savefig('model.png')
︡f158de2d-c514-4999-b457-a5a4dd70cd60︡︡
︠f93f2c11-948a-4695-a21e-e6f3348c64e6︠
alpha = .5
sp = plot(-2*sqrt(-2*x^2 + 5),(-2,2)) + plot(2*sqrt(-2*x^2 + 5),(-2,2)) + plot(4*alpha/x^2,(-2,2))
sp.show(ymax=5,xmin=-1.6,xmax=1.6)
2 != 3
︡793e5998-16f3-4a2c-9b8c-ef92ada980ae︡{"stdout": "verbose 0 (3052: plot.py, generate_plot_points) WARNING: When plotting, failed to evaluate function at 44 points.\nverbose 0 (3052: plot.py, generate_plot_points) Last error message: ''\nverbose 0 (3052: plot.py, generate_plot_points) WARNING: When plotting, failed to evaluate function at 42 points.\nverbose 0 (3052: plot.py, generate_plot_points) Last error message: ''"}︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡{"html": "<span class=\"math\">\\newcommand{\\Bold}[1]{\\mathbf{#1}}{\\rm True}</span>"}︡
︠63008e92-f863-431e-ac81-e6359f188fd4︠
Z = matrix([[8,0],[0,0]])
I0 = [0,0,0,0]

def testsys(t,y):

    if t>55 and t<65:
        boxcar = 1
    else:
        boxcar = 0
        
    dy1 = (1-y[0]**2-y[1]**2)*y[0] -1/16*( (y[1]**2-4)*Z[0,1] + (y[0]**2-4)*Z[0,0] + 16)*y[0]/2 + sqrt(2)*I0[0]/2 + sqrt(2)*I0[2]/2 - 4*boxcar
    dy2 = (1-y[0]**2-y[1]**2)*y[1] -1/16*( (y[1]**2-4)*Z[1,1] + (y[0]**2-4)*Z[1,0] + 16)*y[1]/9 + sqrt(2)*I0[1]/2 + sqrt(2)*I0[3]/2 + 2*boxcar
    
    return [dy1,dy2]
︡b94bee73-2b3d-4e5f-a041-85aa5338d077︡︡
︠c610f378-af00-401a-b84b-0681ea13de35︠
import pylab as p
test = ode_solver( function=testsys, y_0=[.3,.3] )
test.ode_solve( t_span=(0,100), num_points=2000 )

sol = []
t,y = test.solution[0]
sol.append([])
for i in y:
    sol.append([])
for t,y in test.solution:
    sol[0].append(t)
    for j in range(0,len(y)):
        sol[j+1].append(y[j])

import numpy as n
X,Y = n.meshgrid( n.linspace(-1.5,1.5,20), n.linspace(-1.5,1.5,20) )
U,V = n.zeros([20,20]), n.zeros([20,20])
for i in range(0,20):
    for j in range(0,20):
        vij = testsys(0,[X[i,j],Y[i,j]])
        U[i,j] = vij[0]
        V[i,j] = vij[1]
        
p.close()
p.quiver(X,Y,U,V)
p.plot(sol[1],sol[2])
p.savefig("pplane.png")
︡6589cc0c-8526-41ae-b91a-e59392af3475︡︡
︠e9861ee3-6573-47c9-893a-db005d3eafea︠
ode_solver?
︡b691e1e0-a5b7-438d-ab51-0181219bcda5︡︡
